Attribute VB_Name = "mdlDriveFunctions"
' Types
    Public Type tVolumeInfo
        VolumeName As String
        FileSystem As String
        SystemFlags As Long
        ComponentLength As Long
    End Type
    
    Public Type tCurrentFolder
        Drive As String
        Folder As String
    End Type
    
' Consts
' For Get VolumeInformation Flags
    Private Const FS_CASE_IS_PRESERVED = &H2
    Private Const FS_CASE_SENSITIVE = &H1
    Private Const FS_UNICODE_STORED_ON_DISK = &H4
    Private Const FS_PERSISTENT_ACLS = &H8
    Private Const FS_FILE_COMPRESSION = &H10
    Private Const FS_VOL_IS_COMPRESSED = &H8000

'GetDriveType Return Values
'0=Can't Identify drive (error)
'1=Drive Not Present
'2=Removable (Floppy disk or Zip drive type)
'3=Fixed (Hard Drive or Non-Removable Zip drive)
'4=Remote (Network Drive)
'5=CDROM
'6=RAMDISK
' Public Functions
    Public Declare Function GetDriveType Lib "kernel32" Alias "GetDriveTypeA" (ByVal nDrive As String) As Long
    Public Declare Function GetCurrentDirectory Lib "kernel32" Alias "GetCurrentDirectoryA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
    Public Declare Function SetCurrentDirectory Lib "kernel32" Alias "SetCurrentDirectoryA" (ByVal lpPathName As String) As Long
' Private Functions
    Private Declare Function GetLogicalDriveStrings Lib "kernel32" Alias "GetLogicalDriveStringsA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
    Private Declare Function GetVolumeInformation Lib "kernel32" Alias "GetVolumeInformationA" (ByVal lpRootPathName As String, ByVal lpVolumeNameBuffer As String, ByVal nVolumeNameSize As Long, lpVolumeSerialNumber As Long, lpMaximumComponentLength As Long, lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, ByVal nFileSystemNameSize As Long) As Long

    Private Declare Function PathStripToRoot Lib "shlwapi.dll" Alias "PathStripToRootA" (ByVal pszPath As String) As Long
    Private Declare Sub PathStripPath Lib "shlwapi.dll" Alias "PathStripPathA" (ByVal pszPath As String)

Public Function GetCurrentFolder() As tCurrentFolder
    Dim sBuffer As String
    Dim lRet As Long
    
    sBuffer = String(255, Chr(0))
    
    lRet = GetCurrentDirectory(255, sBuffer)
    If lRet > 0 Then
    ' Remove trailing
        sBuffer = Left(sBuffer, lRet)
    
        GetCurrentFolder.Drive = Left(sBuffer, 3)
        If Len(sBuffer) > 3 Then ' May be root drive
            GetCurrentFolder.Folder = Right(sBuffer, Len(sBuffer) - 3)
        End If
    End If
End Function

Public Function FixDrive(ByVal Drive As String) As String
' Make only 1 char long (easier)
    Drive = Left(Drive, 1)
' Make numeric letter
    If IsNumeric(Drive) Then
        Drive = Chr(Drive + 65) '0+17=A
    End If
' Make proper format
    FixDrive = Drive & ":\"
End Function
    
Public Function ExistingDrives() As String
    Dim sTmp As String
    
    sTmp = String(255, Chr(0))
    
    GetLogicalDriveStrings 255, sTmp

' OK, now sTmp = all the drives followed by a null
    sTmp = Left(sTmp, InStr(1, sTmp, Chr(0) & Chr(0)))
    
    ExistingDrives = sTmp
End Function

Public Function DriveType(ByVal DriveLetter As String) As String
    Dim iRet As Integer
    
    DriveLetter = FixDrive(DriveLetter)
' Ok, now get the type
    iRet = GetDriveType(DriveLetter)
    Select Case iRet
    Case 0
        DriveType = "Error"
    Case 1
        DriveType = "Drive Doesn't Exist"
    Case 2
        DriveType = "Removable"
    Case 3
        DriveType = "Fixed (Hard) Drive"
    Case 4
        DriveType = "Network Drive"
    Case 5
        DriveType = "CD-ROM"
    Case 6
        DriveType = "RAM Disk"
    Case Else
        Debug.Print iRet
        DriveType = "Unrecognized Type"
    End Select
End Function

Public Function GetVolumeInfo(ByVal Drive As String) As tVolumeInfo
    Dim lngSerial As Long
    Dim strVolName As String
    Dim strFileSystem As String
    Dim lngComponentlength As Long
    Dim lngFSFlags As Long
    Dim lngRet As Long
    
' Make sure the drive's in the proper format
    Drive = FixDrive(Drive)
    
' Init the vars
    lngSerial = 0
    strVolName = String(255, Chr(0))
    strFileSystem = String(255, Chr(0))

' Get the info
    lngRet = GetVolumeInformation(Drive, strVolName, 255, lngSerial, lngComponentlength, lngFSFlags, strFileSystem, 255)
    
    If lngRet > 0 Then
        GetVolumeInfo.VolumeName = Left(strVolName, InStr(1, strVolName, Chr(0)) - 1)
        GetVolumeInfo.FileSystem = Left(strFileSystem, InStr(1, strVolName, Chr(0)) - 1)
        GetVolumeInfo.SystemFlags = lngFSFlags
        GetVolumeInfo.ComponentLength = lngComponentlength
    End If
End Function
